'
' 4-Wire Resistive Touchscreen Macros
' Touchscreen mounted on a 128x64 KS0108 Graphical LCD.
'
' The 4-Wire Touchscreen is connected directly to AN0, AN1, AN2, and AN3 of the PICmicro
' With 10K Ohm pull-down resistors on each ADC input.
'
' Touchscreen Cable Configuration:-
'
'    X1 ---------\
'                 \
'    Y1 ---------\ \------O To RA0 (AN0)
'  From           \-------O To RA1 (AN1)
'  Touchscreen    /-------O To RA2 (AN2)
'    X2 ---------/ /------O To RA3 (AN3)
'                 /
'    Y2 ---------/
'
' Define some global variables used by the Touchscreen subroutines
'
    Dim wXminimum As Word System
    Dim wYminimum As Word System
    Dim wYmaximum As Word System
    Dim bXrange As Word System
    Dim wYrange As Word System
    Dim bXlocation As Byte System
    Dim wYlocation As Byte System
    Dim bXprevious As Byte System
    Dim wYprevious As Byte System
    Dim bXposResult As Word System
    Dim bYposResult As Word System

    Dim HotSpotCoords As Dword System
    Dim CoordsXStart As HotSpotCoords.Byte0
    Dim CoordsYStart As HotSpotCoords.Byte1
    Dim CoordsXEnd As HotSpotCoords.Byte2
    Dim CoordsYEnd As HotSpotCoords.Byte3
    Dim BlinkFlag As Bit

    Dim wProd As Word
'
'----------------------------------------------------------------------------------------------
' Eeprom Data storage
'
Calibrated_Data EData As Byte 0
wXminimum_Data  EData As Word 0
wYminimum_Data  EData As Word 0
wYmaximum_Data  EData As Word 0
bXrange_Data    EData As Word 0
wYrange_Data    EData As Word 0

'---------------------------------------------------------------------------------------------
' Initialise the ADC and Ports before entering the main program
' This section may require changing for different PICmicro devices
' Always check the datasheet for the appropriate device.
'
Initialise:
    All_Digital = True                            ' Make sure the rest of the analoque pins are digital mode first
    ADCON0 = %11000001                            ' 1 \ FRC (clock derived from the internal ADC RC oscillator)
                                                  ' 1 /
                                                ' 0 \
                                                ' 0   Analog Channel 0 (AN0) Select
                                                ' 0 /
                                                ' 0 - GO_DONE: A/D Conversion clear
                                                ' 0 - Unimplemented
                                                ' 1 - A/D Converter module is powered up

    ADCON1 = %10000010                            ' 1 - Right justified (Six Most Significant bits of ADRESH are read as 0)
                                                 ' 0 - Clear A/D Conversion Clock Select bit
                                                ' 0 \
                                                ' 0 / Unimplemented
                                                ' 0 \
                                                ' 0   ADC Port Configuration Control
                                                ' 1   All PORTA Analogue, PORTE Digital
                                                ' 0 /
    TRISA = 0                                    ' PortA is Output
    LATA = 0                                    ' PortA = Low
    GoTo OverTouchScreenSubs                    ' Jump over the subroutines

'----------------------------------------------------------------------------------------------
' Calibrate the Touch Screen
' Flashes a cursor Top-Left then Bottom-Right of the screen
' When each cursor appears, the stylus must be placed upon it
'
' Syntax:    CalibrateScreen()

$define CalibrateScreen() GoSub _CalibrateScreen
'
' Input        : None
' Output    : wXminimum, wYminimum, wYmaximum, bXrange, wYrange
' Notes        : Stores the above values into eeprom once the calibration has taken place

_CalibrateScreen:
    BlinkFlag = 1
    Repeat
        GoSub _GetRawXpos                        ' Wait for stylus release
    Until bXposResult <= 1
    DelayMS 30                                  ' Debounce delay
    Repeat
        If BlinkFlag = 1 Then                    ' Flash a cursor in the top-left corner
            LCDWrite 0,0,[15,15,15,15]
        Else
            LCDWrite 0,0,[0,0,0,0]
        EndIf
        GoSub _GetRawXpos                          ' Wait for the stylus to touch the screen
        DelayMS 300
           BlinkFlag = ~BlinkFlag
    Until bXposResult > 0
    GoSub _GetRawYpos
    wYminimum = bYposResult                         ' Save Y ADC value as minimum
    GoSub _GetRawXpos
    wXminimum = bXposResult                          ' Save X ADC value as minimum
    LCDWrite 0,0,[0,0,0,0]                        ' Remove the top-left cursor
    Repeat
        GoSub _GetRawXpos                       ' Wait for stylus release
    Until bXposResult <= 1
    DelayMS 30                                  ' Debounce delay
    Repeat
           If BlinkFlag = 1 Then                    ' Flash a cursor in the bottom-right corner
            LCDWrite 7,124,[240,240,240,240]
        Else
            LCDWrite 7,124,[0,0,0,0]
        EndIf
       GoSub _GetRawXpos                        ' Wait for the stylus to touch the screen
       DelayMS 300
       BlinkFlag = ~BlinkFlag
    Until bXposResult > 0
    GoSub _GetRawYpos
    wYmaximum = bYposResult                        ' Save Y ADC value as maximum
    wYrange = wYmaximum - wYminimum                 ' Calculate Y ADC range across the LCD
    GoSub _GetRawXpos
    bXrange = bXposResult - wXminimum               ' Calculate X ADC range across the LCD
    LCDWrite 7,124,[0,0,0,0]                    ' Remove the bottom-right cursor
    Repeat
        GoSub _GetRawXpos                          ' Wait for stylus release
    Until bXposResult = 0
'
' Write the calibration values to the on-board eeprom
'
    EWrite Calibrated_Data,[1]
    EWrite wXminimum_Data,[wXminimum]
    EWrite wYminimum_Data,[wYminimum]
    EWrite wYmaximum_Data,[wYmaximum]
    EWrite bXrange_Data,[bXrange]
    EWrite wYrange_Data,[wYrange]
    Return


'----------------------------------------------------------------------------------------------
' Get the Raw X position from the ADC
'
' Input        : None
' Output    : Word "bXposResult" holds the modified ADC value
'
_GetRawXpos:
    TRISA = TRISA & %11110000                       ' Mask and set to output ADC pins
    TRISA = TRISA | %00000010                       ' RA1(AN1) is input
    LATA = %00000101                                ' Energise the Y plate and read the X plate
    bXposResult = ADIn 1                            ' Read the voltage drop on Y+
    If bXposResult > 0 Then                         ' Is the stylus down?
        TRISA.1 = 0                                 ' Yes. So RA1(AN3) is output
        TRISA.3 = 1                                 ' RA3(AN1) is Input
        wProd = 1023 - ADIn 3                       ' Read the voltage drop from the opposite side
        bXposResult = bXposResult + wProd           ' \
        bXposResult = bXposResult / 2               ' / Get the average reading
    EndIf
    Return

'----------------------------------------------------------------------------------------------
' Get the Raw Y position from the ADC
'
' Input     : None
' Output    : Word "bYposResult" holds the modified ADC value
'
_GetRawYpos:
    TRISA = TRISA & %11110000                       ' Mask and set to output ADC pins
    TRISA = TRISA | %00000100                       ' RA2(AN2) is input
    LATA = %00001010                                ' Energise the X plate and read the Y plate
    bYposResult = ADIn 2                            ' Read the voltage drop on X-
    If bYposResult > 0 Then                         ' Is the stylus down?
       TRISA.2 = 0                                  ' Yes. So RA2(AN2) is output
        TRISA.0 = 1                                 ' RA0(AN0) is input
        wProd = 1023 - ADIn 0                       ' Read the voltage drop from the opposite side
        bYposResult = bYposResult + wProd           ' \
        bYposResult = bYposResult / 2               ' / Get the average reading
    EndIf
    Return

'----------------------------------------------------------------------------------------------
' Test for the stylus touching the LCD and calculate the stylus position
' Input     : None
' Output    : WREG is 1 if the stylus is touching the LCD, else 0
'           : Byte "bXlocation" holds the X position of the stylus
'           : Byte "wYlocation" holds the Y position of the stylus
'
' Syntax:
' Var = LookForStylus
'
' Will return a value of 1 if the stylus is touching the screen, else returns 0
'
$define LookForStylus() _mLookForStylus

_mLookForStylus Macro
    GoSub _LookForStylus
    #if(_mLookForStylus_Return == 1)                        ' Is there a return value
        #if(Return_Type == Bit)
            Wreg_Bit Return_Var,Return_Bit
        #endif
        #if(Return_Type == Byte)
            #if(Return_Var != WREG)
                Wreg_Byte Return_Var
            #endif
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
    #endif
    Endm

_LookForStylus:
    GoSub _GetRawXpos                                   ' Get the raw X ADC value
    If bXposResult > 0 Then                             ' Is the stylus down?
        DelayMS 30                                      ' Yes. So debounce delay
        GoSub _GetRawYpos                               ' Get the raw Y ADC value
        GoSub _GetRawXpos                               ' Get the raw X ADC value
        If bYposResult > wYminimum Then                 ' \ Is the stylus within the Y range of the screen?
            If bYposResult < wYmaximum Then             ' /
                wProd =  bXposResult - wXminimum        ' \
                wProd = wProd << 7                      '   Calculate the stylus X location (n * 128)
                bXlocation = wProd / bXrange            ' /
                If bXlocation > 127 Then bXlocation = 127    ' Keep the stylus within the LCD's X boundary
                wProd = bYposResult - wYminimum         ' \
                wProd = wProd << 6                      '   Calculate the stylus Y location (n * 64)
                wYlocation = wProd / wYrange            ' /
                If wYlocation > 63 Then wYlocation = 63 ' Keep the stylus within the LCD's Y boundary
                WREG = 1                                ' Indicate Stylus is down
            EndIf
        EndIf
    Else
        WREG = 0
    EndIf
    Return

'----------------------------------------------------------------------------------------------
' Check of the Stylus is within a certain area of the LCD
' Create a macro to pass the coordinates to the subroutine and return a value
'
' Syntax:
' Var = Hotspot Top-Left X coord, Top-Left Y coord, Bottom-Right X coord, Bottom-Right Y coord
'
' pXposStart, pYposStart
'           X------------------------
'           |                       |
'           |                       |
'           |                       |
'           |                       |
'           ------------------------X
'                           pXposEnd, pYposEnd
'
' Will return 1 if the stylus is located within the area designated
'
$define HotSpot(pXS, pYS, pXE, pYE) _mHotSpot pXS, pYS, pXE, pYE

_mHotSpot Macro pXposStart, pYposStart, pXposEnd, pYposEnd
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
        Num_Byte pXposStart , CoordsXStart
    #endif
    #if((Prm_1 == Byte) || (Prm_1 == Word) || (Prm_1 == Dword))
        Byte_Byte pXposStart , CoordsXStart
    #endif

    #if((Prm_2 == Num8) || (Prm_2 == Num16) || (Prm_2 == Num32))
        Num_Byte pYposStart , CoordsYStart
    #endif
    #if((Prm_2 == Byte) || (Prm_2 == Word) || (Prm_2 == Dword))
        Byte_Byte pYposStart , CoordsYStart
    #endif

    #if((Prm_3 == Num8) || (Prm_3 == Num16) || (Prm_3 == Num32))
        Num_Byte pXposEnd , CoordsXEnd
    #endif
    #if((Prm_3 == Byte) || (Prm_3 == Word) || (Prm_3 == Dword))
        Byte_Byte pXposEnd , CoordsXEnd
    #endif

    #if((Prm_4 == Num8) || (Prm_4 == Num16) || (Prm_4 == Num32))
        Num_Byte pYposEnd , CoordsYEnd
    #endif
    #if((Prm_4 == Byte) || (Prm_4 == Word) || (Prm_4 == Dword))
        Byte_Byte pYposEnd , CoordsYEnd
    #endif
    GoSub _HotSpot
    #if(_mHotSpot_Return == 1)                    ' Is there a return value
        #if(Return_Type == Bit)
            Wreg_Bit Return_Var,Return_Bit
        #endif
        #if(Return_Type == Byte)
            #if(Return_Var != WREG)
                Wreg_Byte Return_Var
            #endif
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
    #endif
    Endm
'
' Input     : Byte "CoordsXStart" is the Top-Left X coordinate of the area to check
'           : Byte "CoordsYStart" is the Top-Left Y coordinate of the area to check
'           : Byte "CoordsXEnd" is the Bottom-Right X coordinate of the area to check
'           : Byte "CoordsYEnd" is the Bottom-Right Y coordinate of the area to check
' Output    : WREG = 1 if Stylus is within the area, else 0
'
#ifdef _mHotSpot#Req
_HotSpot:
    If bXlocation >= CoordsXStart Then
        If wYlocation >= CoordsYStart Then
            If bXlocation <= CoordsXEnd Then
                If wYlocation <= CoordsYEnd Then
                    WREG = 1
                    Return
                EndIf
            EndIf
        EndIf
    EndIf
    WREG = 0
    Return
#endif

'----------------------------------------------------------------------------------------------
OverTouchScreenSubs:

